/*
 * Copyright (c) 2022 Contributors to the Eclipse Foundation.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *   Data In Motion - initial API and implementation 
 */
package org.eclipse.sensinact.model.core.provider;

import org.eclipse.emf.common.util.EMap;

import org.eclipse.emf.ecore.EClass;

import org.osgi.annotation.versioning.ProviderType;

/**
 * <!-- begin-user-doc -->
 * A representation of the model object '<em><b>Dynamic Provider</b></em>'.
 * <!-- end-user-doc -->
 *
 * <!-- begin-model-doc -->
 * To achieve a good Model, concrete EClasses that inherit from Provider should 
 * use References for predefined Services.
 * Sometimes however you can't know how many Services a Provider can have. 
 * In such a case, this Map can be used in addtion to directly Referenced Services.
 * 
 * As an example: Such a Provider can be a Camera that can detect Objects like cars, 
 * bikes or pedestriance in configureable zones. Each zone can in this case be a 
 * Service with defined Resources. How may zones exist however, depdends on the 
 * configuration. Thus our camera model can benefit from service map in addition
 * some predefined fix services it might have.
 * 
 * Service names must still be unique and must not collide with existing References.
 * 
 * <!-- end-model-doc -->
 *
 * <p>
 * The following features are supported:
 * </p>
 * <ul>
 *   <li>{@link org.eclipse.sensinact.model.core.provider.DynamicProvider#getServices <em>Services</em>}</li>
 * </ul>
 *
 * @see org.eclipse.sensinact.model.core.provider.ProviderPackage#getDynamicProvider()
 * @model
 * @generated
 */
@ProviderType
public interface DynamicProvider extends Provider {
	/**
	 * Returns the value of the '<em><b>Services</b></em>' map.
	 * The key is of type {@link java.lang.String},
	 * and the value is of type {@link org.eclipse.sensinact.model.core.provider.Service},
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * <!-- begin-model-doc -->
	 * To achieve a good Model, concrete EClasses that inherit from Provider should 
	 * use References for predefined Services.
	 * Sometimes however you can't know how many Services a Provider can have. 
	 * In such a case, this Map can be used in addtion to directly Referenced Services.
	 * 
	 * As an example: Such a Provider can be a Camera that can detect Objects like cars, 
	 * bikes or pedestriance in configureable zones. Each zone can in this case be a 
	 * Service with defined Resources. How may zones exist however, depdends on the 
	 * configuration. Thus our camera model can benefit from service map in addition
	 * some predefined fix services it might have.
	 * 
	 * Service names must still be unique and must not collide with existing References.
	 * 
	 * <!-- end-model-doc -->
	 * @return the value of the '<em>Services</em>' map.
	 * @see org.eclipse.sensinact.model.core.provider.ProviderPackage#getDynamicProvider_Services()
	 * @model mapType="org.eclipse.sensinact.model.core.provider.ServiceMap&lt;org.eclipse.emf.ecore.EString, org.eclipse.sensinact.model.core.provider.Service&gt;"
	 * @generated
	 */
	EMap<String, Service> getServices();

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @model annotation="http://www.eclipse.org/emf/2002/GenModel body='Service service = super.getService(serviceName);\nreturn service != null ? service : getServices().get(serviceName);'"
	 * @generated
	 */
	Service getService(String serviceName);

	/**
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @model annotation="http://www.eclipse.org/emf/2002/GenModel body='EClass serviceType = super.getServiceEClass(serviceName);\nif (serviceType != null) {\n    return serviceType;\n} else {\n    Service service = getServices().get(serviceName);\n    return service != null ? service.eClass() : null;\n}'"
	 * @generated
	 */
	EClass getServiceEClass(String serviceName);

} // DynamicProvider
